---
layout: workbook
title: GitHub Foundations Workbook
description: This student and teacher workbook will be your companion to the GitHub Foundations class taught by the GitHub Training Team and other educational groups. In this course, you'll learn basic collaboration skills towards a productive use of Git and GitHub in your open source work and daily job assignments.


---

## Version control concepts

### Summary
* Git, the information tracker
* GitHub, the collaboration platform
* Distributed version control
* <a href="http://git-scm.com/book/en/Getting-Started-A-Short-History-of-Git" class="booklink">A short history of Git</a>
* <a href="http://git-scm.com" class="weblink">The Git project homepage</a>

#### Details
__Git is an open source distributed__ version control system invented by Linus Torvalds in 2005.  It is used to version the Linux kernel and is shown to be, by some research, the most popular modern version control system.

Git is the open source, command line tool that forms a part of the workflows we'll cover in this course.  It has been called the successor to CVS and Subversion by many users, and maintains some similarity in behavior to these historical open source version control tools.

Maintaining a current version of Git is important. While you'll accurately hear that Git is generously backwards-compatible, the latest versions offer error message enhancements, performance tunings, and usability features that make using Git ever more quick and pleasant.

Verifying that Git is installed and operational can be done by requesting Git to display its current version using this command:

``` shell
$ git --version

git version 1.8.2
```

##  GitHub Installers
* <a href="http://windows.github.com" class="weblink">GitHub for Windows installer</a>
* <a href="http://mac.github.com" class="weblink">GitHub for Mac installer</a>
* <a href="http://git-scm.com/book/en/Getting-Started-Installing-Git" class="booklink">Pro Git Book: Installing Git</a>
* **Windows**: The GitHub for Windows installer can be found at http://windows.github.com.
* **Mac**: The GitHub for Mac installer can be found at http://mac.github.com.
* **Linux**: Git's source code and a listing of supported package managers can be found at http://git-scm.com/download/linux.

### Details
The command line version of Git has a very light footprint. For most platforms, you can simply copy the binaries to a folder that is on the executable search $PATH. Git is primarily written in C, which means there is a unique installer for each operating system.


## The GitHub platform
*  <a href="https://github.com/features" class="githublink">GitHub.com feature list</a>
* <a href="https://www.youtube.com/watch?v=ezxRcdJ8glM&list=PLg7s6cbtAD17rhrz2BJWAPJMjR71B3IDx" class="videolink">How to sign up for GitHub</a>

### Details
GitHub accounts are free. Sign up for one at [github.com/join](https://github.com/join).

GitHub is the repository hosting, collaboration, deployment, and distribution platform for both open source and private software projects.  It facilitates discussing changes-in-progress through the concepts of Issues and Pull Requests and provides a web user interface to much of Git. The web flow made possible by GitHub brings a much wider range of contributors, including documentation specialists, designers, and ops engineers into the flow of contributing to your application's life cycle.

GitHub accounts are free for an unlimited quantity of public repositories. Only private repositories cost to host on the service. Private repositories are typically used for closed-source consulting client or corporate products, whereas public repositories are typically used for open source.

Sign up for a free GitHub account at https://github.com/join.

Git can be installed as a unified GitHub GUI and command line or merely as a stand-alone command line interface.


## Ways of interacting with repositories

### Summary
* <a href="https://guides.github.com/overviews/flow/" class="booklink">GitHub web flow</a>
* GitHub GUIs for Mac, Windows
* Command line

## Getting to know GitHub

### Summary
* Repository initialization
* Actionable steps (Create, Edit, Remove, Move)
* Commits (page)

## Version control basics

### Summary
* Commits
  * Line-level tracking
  * Commit messages
  * Grouping files (with desktop clients)
* Branching
  * Branching concepts
  * Branch selection menu
  * Branch listing page
  * Branch comparison view

## Acquiring repos

### Summary
* Clone from web (Clone in Desktop button)
* Clone via personal repo list in GitHub Desktop
* Complete "copy" explanation

## Local repository interaction

### Summary
* Changes (with text editor)
* Comparison (diff via GHfD)
* Commits
* Branches
* History
* Syncing (GHfD)

## GitHub workflows

### Summary
* Forking
    * <a href="https://guides.github.com/overviews/forking/" class="booklink">Guide to Forking</a>
    * Reason for forks
    * What it encourages
    * How it insulates
    * Keeping up to date (too advanced?)
    * Original (base) and Fork (individual copy)
* The GitHub Flow
    * Branch
    * Commit
    * Compare
    * Pull Request
    * Discuss
    * Animated GIF
    * Line by line code review conversation
    * Clone locally with GHfD (optional)

## GitHub visualizations

### Summary
* Comparing/Diff-ing
  * Code
  * Prose
  * Images
  * Maps
  * Spreadsheets
  * 3D models
* Branch view
* Branch ahead/behind positions
* Network graph
* Activity graph

## Project management basics

### Summary
* GitHub Issues
* Milestones
* Pulse
* Tags, Releases
